#!/usr/bin/perl -w
use diagnostics ;
use strict;

use DBI ;

package ezApp::DB ;		# DB handles the interface to DBI.
					
#-------------------------------------------------------------------
#
# 	Copyright (c) 2001 Richard K. Moore.
#				richard@cyberjournal.org
#
#	This script is free software; you can redistribute it and/or
#	modify it under the same terms as Perl itself.
#
#-------------------------------------------------------------------
#
#	Except for &new(), failures always return '', and successful results always test as true.
#	Diagnostic messages are sent to $Job->Warn().
#		
#-----------------------------------------------------------------------------------------------------------
sub new 				# Open our database and  return self.
								# Set $self->{isopen} to false if warnings, else true.
{ 
	my ( $class, $job , $dbname, $user, $handler, $password) = map {$_ ? $_ : ''} @_ ;
	my $self = 	{ job=>$job, dbname=>$dbname } ;
	bless $self, $class ;
	
#	die "DB new: '$handler','$dbname','$user','$password'\n";
#	$user			||= 'root';
#	$handler 	||= 'mysql';
#	$password ||= undef;

# if "The DBI access parameters for the database" are not passed correctly from ../ccConfig.pm 
# then try resetting them here as required.

#  $user     = 'root';
#  $handler  = 'mysql';
#  $password = undef;
#  $dbname = 'dbname';	
	
	
	my $dbh = DBI->connect("DBI:$handler:$dbname", $user, $password);
	if ($dbh)
	{
		$self->{dbh} = $dbh;
		$self->{isopen}++
	}
	else
	{
		$password ||= 'undef';
		$job->Warn
			("Failed to open database.\n\$dbh = DBI->connect('DBI:$handler:$dbname', $user, $password);")
	}
$self}
#-----------------------------------------------------------------------------------------------------------
sub Close 			# Disconnect from database.
{ 
	my $self = shift ;
	if ( $self->{isopen} )
	{
		my $dbh = $self->{dbh} ;
		$dbh->disconnect;
		$self->{isopen}--
	}
	else
		{ $self->{job}->Warn("Cannot close database $self->{dbname} - it is not currently open.") }
}
#-----------------------------------------------------------------------------------------------------------
sub Rows				# Return specified array of rows.
								# Call: Records		( 'TableName', 
								#										-Columns 	=> [qw(colName colName)], # at least one col name required.
								#										-Where   	=>'condition', 						# default ''		eg: 'RecNum=7425'
								#										-OrderBy 	=>'order clause',					# default ''		eg: 'Count>50'
								#										-Limit   	=>'number' 								# default ''		eg: '10'
								#									)  or ERROR ;
{
	my ( $self, $TableName, @OtherArgs ) = @_ ;
	my $dbh 				= $self->{dbh} ;
	my $job 				= $self->{job} ;
	my $Dispatcher 	= $job->{Dispatcher};
		
	my $Args = $job->ArgsAsHash( [], @OtherArgs )
		|| return $Dispatcher->dieDispatcher
			("Job->ArgsAsHash returned undefined value to DB->Rows");
	my ( $Columns, $Where, $OrderBy, $Limit ) = map {$_ ? $_ : ''} @$Args{ qw( Columns Where OrderBy Limit ) } ;
	( $Columns && @$Columns ) || return $job->Warn('No Columns supplied to DB->Rows') ;
	my $ColList = join ',', @$Columns ;
	
	my $Query = "select $ColList from $TableName"
		.( $Where ? " where $Where" : '' )
		.( $OrderBy ? " order by $OrderBy" : '' )
		.( $Limit ? " limit $Limit" : '' ) ;
	
	my ( $Sth, $Rv ) ;
	my $Rows = [] ;
	unless	( 	( $Sth = $dbh->prepare($Query) ) && 
				( $Rv = $Sth->execute ) &&
				( $Rows = $Sth->fetchall_arrayref ) 
			) {
		my $Message = $dbh->errstr ;
		return $job->Warn
			("SQL error in database '$self->{dbname}'\n$Message\nQuery = '$Query'") ; 
	} ;
	$self->{Columns} = $Columns  ;
$Rows }
#-----------------------------------------------------------------------------------------------------------
sub Records			# Return specified array of records as hahes, fieldName => fieldValue .
								# Call: Records		( 'TableName', 
								#										-Columns 	=> [qw(colName colName)], # at least one col name required.
								#										-Where   	=>'condition', 						# default ''		eg: 'RecNum=7425'
								#										-OrderBy 	=>'order clause',					# default ''		eg: 'Count>50'
								#										-Limit   	=>'number' 								# default ''		eg: '10'
								#									)  or ERROR ;
{
	my $self 				= shift;
	my $Dispatcher 	= $self->{Dispatcher};
	my $Rows;
	$Rows = $self->Rows(@_);
	defined $Rows
		|| $Dispatcher->dieDispatcher
			("DB->Rows returned undefined result to DB->Records");
	$Rows
		|| return '';
	my $Records = [] ;
	my @Columns = @{ $self->{Columns} };
	foreach my $Row (@$Rows) 
	{
		push @$Records, { map { $_ => shift @$Row } @Columns }
	} ;
$Records}
#-----------------------------------------------------------------------------------------------------------
sub DoSQL { my ($self, $Statement ) = @_ ;							# Perform specified SQL statement and return its result.
	my $dbh = $self->{dbh} ;
	my $job = $self->{job} ;
	my $Result = $dbh->do($Statement) ;
	if ( ! $Result ) {
		my $Message = $dbh->errstr ;
		return $job->Warn
			("SQL error in database '$self->{dbname}'\nStatement = '$Statement'\n$Message") ;
	} ;
$Result}
#-----------------------------------------------------------------------------------------------------------
sub UpdateRecords	# Update records as specifed.
									#
									# Call: UpdateRecords	( 'TableName', 	
									#								-Columns 	=> [qw(colName colName)], 	# at least one col name required
									#								-Values => ['value', 'value']					# same length as Columns, values quoted for SQL
									#								-Where  => "condition" 								# required - eg: 'AcctNum=7425'
									#							) or ERROR ;
{
	my ( $self, $TableName, @OtherArgs ) = @_ ;
	my $job = $self->{job} ;
	my $dbh = $self->{dbh} ;

	my $Args = $job->ArgsAsHash( [], @OtherArgs ) ;
	my ( $Columns, $Values, $Where ) = map {$_ ? $_ : '' } @$Args{ qw( Columns Values Where ) } ;
	( $Columns && @$Columns && $Values && $Where )
		|| return $job->Warn('Missing arguments in call to DB->UpdateRecords') ;
	@$Columns == @$Values
		|| return $job->Warn
			('Columns and Values are not the same length in call to DB->UpdateRecords') ;
	my @Assignments = map { "$_=" . shift @$Values } @$Columns ;
	my $SetClause = join ', ', @Assignments ;
	my $SQL = "update $TableName set $SetClause where $Args->{Where}" ;
	$self->DoSQL($SQL)
		|| return $job->Warn('Update failed') ;
1}
#-----------------------------------------------------------------------------------------------------------
sub QuotedSQLvalue 					# Return the value quoted for mySQL.
{ 
	my ( $self, $Value ) = @_ ;	
	my $dbh = $self->{dbh} ;
$dbh->quote($Value) } 
#-----------------------------------------------------------------------------------------------------------
sub QuotedSQLlist 					# Quote each item in the list, join those into a
														# comma-separated string, and return the string.
{ 
	my ( $self, $ValueList ) = @_ ;	
	my $dbh = $self->{dbh} ;
join ', ', map { $dbh->quote($_) } @$ValueList }
#-----------------------------------------------------------------------------------------------------------
sub LastInsertId { my $self = shift ;									# Return last_insert_id for this database handle.
	my $job = $self->{job} ;
	my $Rows = $self->Rows( 'select last_insert_id()' ) 
		or return '' ;
	return $job->Warn("Database '$self->{dbname}' could not find LastInsertId")
		unless @$Rows ;
shift @{ shift @$Rows } }
#-----------------------------------------------------------------------------------------------------------
sub InsertCommas		# Insert commas into numbers for readability.
										# Return the value of the first converted number.
										# ShowZeroAs is the result desired for zero values (usually '' or 0).
										# If number is negative, show it as '(absvalue)'.
{ 
	my ( $self, $ShowZeroAs, @NumRefs ) = @_;
	foreach my $NumRef (@NumRefs) 
	{
		my ( $Num, $Segment, @Segments ) = ( abs($$NumRef), 0, () );
		if ( ! $Num ) 
		{
			$$NumRef = $ShowZeroAs;
		} 
		else 
		{
			while ($Num) 
			{
				my $Segment = $Num % 1000;
				$Num = int ( $Num/1000 );
				$Segment = substr( "00$Segment" , -3, 3 ) if $Num;
				unshift @Segments, $Segment;
			};
			$Num = join ',', @Segments;
			$$NumRef = $$NumRef >= 0 ? $Num : "($Num)";
		};
	};
${$NumRefs[0]} }
#-----------------------------------------------------------------------------------------------------------
1 ;
#-----------------------------------------------------------------------------------------------------------




